////////////////////////////////////////////////////////////////////////
// Class:       OnlineTDCCluster
// Module Type: producer
// File:        OnlineTDCCluster_module.cc
//
// Generated at Wed Oct 30 13:56:06 2013 by Alessandro Razeto using artmod
// from cetpkgsupport v1_02_00.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDProducer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "artdaq-core/Data/Fragments.hh"
#include "darksidecore/Data/V1495Fragment.hh"
#include "darksidecore/Data/V1190Fragment.hh"
#include "darkmon/ArtModules/OnlineEvent.hh"
#include "darksidecore/Data/Config.hh"

#include "TH1.h"
#include <iostream>

namespace ds50 {
  class OnlineTDCCluster : public art::EDProducer {
    public:
      explicit OnlineTDCCluster(fhicl::ParameterSet const & p);
      virtual ~OnlineTDCCluster();

      void produce(art::Event & e) override;
      void beginRun(art::Run &) override;

    private:
      std::unique_ptr<TH1I> h_;
      int begin_threshold_, end_threshold_;
  };
}

ds50::OnlineTDCCluster::OnlineTDCCluster(fhicl::ParameterSet const & ps): begin_threshold_(ps.get<int> ("begin_threshold")), end_threshold_(ps.get<int> ("end_threshold")) {
   produces<ds50::Online::TDCClusters> ();
}

ds50::OnlineTDCCluster::~OnlineTDCCluster() {}

void ds50::OnlineTDCCluster::produce(art::Event & e) { 
  art::Handle<artdaq::Fragments> v1495;
  e.getByLabel ("daq", "V1495", v1495);
  if (!v1495.isValid ()) return;

  if(v1495->size() != 1) {
    mf::LogError ("OnlineTDCCluster") << v1495->size() << " fragment for V1495";
    return;
  }
  if (V1495Fragment((*v1495)[0]).trigger_type () != V1495Fragment::tpc) return;

  art::Handle<artdaq::Fragments> v1190s;
  e.getByLabel ("daq", "V1190", v1190s);
  if (!v1190s.isValid ()) return;

  std::unique_ptr<ds50::Online::TDCClusters> clusters(new ds50::Online::TDCClusters);
  for (size_t i = 0; i < v1190s->size(); ++i) {
    const auto& frag((*v1190s)[i]);

    float resolution_ns = frag.hasMetadata () ? frag.metadata<V1190Fragment::metadata> ()->resolution_ps * 1e-3 : 0.8;

    V1190Fragment b(frag);
    if (b.dataBegin ()->geo () != Config::V1190_LS_board_id && b.dataBegin ()->geo () != Config::V1190_WC_board_id) continue;

    h_->Reset ();
    for (const V1190Fragment::Word *w = b.dataBegin (); w != b.dataEnd (); ++w) 
      if (w->type () == V1190Fragment::Word::measurement_w && !w->trailing_edge ())
	h_->Fill (w->measurement () * resolution_ns);

    for (int i = 1; i < h_->GetNbinsX (); i++) {
      if ((h_->GetBinContent(i) + h_->GetBinContent(i + 1)) >= begin_threshold_) {
	int start_coarse = i;
	for (; i < h_->GetNbinsX (); i++) if ((h_->GetBinContent(i) + h_->GetBinContent(i + 1)) < end_threshold_) break;

	ds50::Online::TDCCluster current_cluster;
	current_cluster.start_time_ns = 10000000;
	current_cluster.stop_time_ns = -100;
	current_cluster.mean_time_ns = 0;
	current_cluster.n_hits = 0;
	current_cluster.n_long_hits = 0;
	current_cluster.long_hits_avg = 0;

	std::array<short int, 256> multiplicity;

	double start_bin = h_->GetBinLowEdge (start_coarse);
	double stop_bin = h_->GetBinLowEdge (i + 1) + h_->GetBinWidth (1);
	for (const V1190Fragment::Word *w = b.dataBegin (); w != b.dataEnd (); ++w) {
	  if (w->type () != V1190Fragment::Word::measurement_w || w->trailing_edge ()) continue;
	  double t = w->measurement () * resolution_ns;
	  if (t < start_bin) continue;
	  if (t > stop_bin) continue;
	  current_cluster.start_time_ns = std::min (current_cluster.start_time_ns, int(t));
	  current_cluster.stop_time_ns = std::max (current_cluster.stop_time_ns, int(t));
	  current_cluster.n_hits ++;
	  current_cluster.mean_time_ns += t - current_cluster.start_time_ns;
	  multiplicity[w->channel ()] ++;

	  for (const V1190Fragment::Word *w2 = b.dataBegin (); w2 != b.dataEnd (); ++w2) {
	    if (w2->type () != V1190Fragment::Word::measurement_w || !w2->trailing_edge () || w2->channel () != w->channel ()) continue;
	    float t2 = w2->measurement () * resolution_ns;
	    if (t2 <= t || t2 > stop_bin) continue;
	    //float charge = std::max ((t2 - t) / 50.F, 1.F);
	    float dt = t2 - t;
	    if (dt > 20 && dt < 200) {
	      current_cluster.n_long_hits ++;
	      current_cluster.long_hits_avg += dt;
	    }
	    break;
	  }
	}
	current_cluster.mean_time_ns /= current_cluster.n_hits;
	if (current_cluster.n_long_hits > 0) current_cluster.long_hits_avg /= current_cluster.n_long_hits;
	for (size_t c = 0; c < sizeof (current_cluster.multiplicity) / sizeof (current_cluster.multiplicity[0]); c++) current_cluster.multiplicity[c] = std::count (multiplicity.begin (), multiplicity.end (), c);

	if (b.dataBegin ()->geo () == Config::V1190_LS_board_id) clusters->ls_clusters.push_back (current_cluster);
	else clusters->wc_clusters.push_back (current_cluster);
      }
    }
  } 
  
  e.put (std::move (clusters));
}

void ds50::OnlineTDCCluster::beginRun(art::Run &) {
  h_ = std::unique_ptr<TH1I> (new TH1I ("otdc__", "", 2400, 0, 120e3));
}

DEFINE_ART_MODULE(ds50::OnlineTDCCluster)
